@@ -16,6 +16,7 @@ extern "C" {
 
 SEXP make_Polygonlist(SEXP iG, SEXP iGc);
 SEXP make_Polygon(SEXP jG, SEXP jGc);
+SEXP rgdal_sp_linkingTo_version(void);
 
 void installErrorHandler();
 void uninstallErrorHandlerAndTriggerError();
@@ -89,7 +90,7 @@ SEXP RGDAL_CPLSetConfigOption(SEXP inOption, SEXP value);
 SEXP RGDAL_CPLGetConfigOption(SEXP inOption);
 SEXP RGDAL_CPL_RECODE_ICONV(void);
 
-SEXP checkCRSArgs(SEXP args, SEXP init_found);
+SEXP checkCRSArgs(SEXP args);
 SEXP PROJcopyEPSG(SEXP tf);
 SEXP ogrInfo(SEXP ogrsourcename, SEXP Layer);
 SEXP R_OGR_types(SEXP dsn, SEXP layer);
@@ -107,6 +108,7 @@ SEXP ogrDeleteDataSource (SEXP ogrSource, SEXP ogrDriver);
 SEXP ogrCheckExists (SEXP ogrSource, SEXP Layer);
 SEXP PROJ4VersionInfo(void);
 SEXP PROJ4NADsInstalled(void);
+SEXP PROJ4_proj_def_dat_Installed(void);
 SEXP transform(SEXP fromargs, SEXP toargs, SEXP npts, SEXP x, SEXP y, SEXP z);
 SEXP projInfo(SEXP type);
 void project(int *n, double *xlon, double *ylat, double *x, double *y,
